#include <Cth.h>

/****************   Rotary Encoder   ****************/
#define inputCLK 4
#define inputDT 5

int counter;
int currentStateCLK;
int previousStateCLK;

/****************   Lichtsensoren   ****************/
float waardeL1;
float waardeL2;
float licht;

/**********   Ultrasone afstandssensor   **********/
float totaldistance;
const int trigPin = 6;
const int echoPin = 7;
float duration, distance;

/*****************   Ventilatoren *****************/
word VentPin = 3;
float prevPWM = -1;
int powerPin = 9;
float minTemp = 30;
float curTemp;
int automatic = 0;



void setup() {
  // put your setup code here, to run once:
  pinMode(inputCLK, INPUT);
  pinMode(inputDT, INPUT);
  pinMode(VentPin, OUTPUT);

  pinMode(trigPin, OUTPUT);
  pinMode(echoPin, INPUT);

  pinMode(powerPin, OUTPUT);
  digitalWrite(powerPin, LOW);

  previousStateCLK = digitalRead(inputCLK);
  Scheduler.startLoop(rotaryEncoder);
  Scheduler.startLoop(loopLight);
  Scheduler.startLoop(distanceToScreen);
  Serial.setTimeout(700);
  Serial.begin(115200);
  pwm25kHzBegin();

}

void loop() {
  // put your main code here, to run repeatedly:

  /*********************************************************************************************************
    INPUT
  *********************************************************************************************************/

  if (Serial.available()) {
    String line = Serial.readString();
    String eenheid = line.substring(0, 1);
    String waardeString = line.substring(1);
    float waarde = waardeString.toFloat();
    if (eenheid == "T") {
      fanActivation(waarde);
      curTemp = waarde;
      if (waarde < minTemp) {
        counter = 0;
      }

      Serial.print("P" + String(counter) + "L" + String(licht) + "D" + String(totaldistance));
    }
    if (eenheid == "S") {
      counter = waarde;
    }

    if (eenheid == "A") {
      automatic = waarde;
    }
    if (eenheid == "M") {
      minTemp = waarde;
      //Serial.print("minTemp aangepast");
    }
  }



  /*********************************************************************************************************
    OUTPUT
  *********************************************************************************************************/

  /*Serial.print("Speed:  ");
    Serial.println(counter);
    Serial.print("Lichtsterkte verschil:  ");
    Serial.println(licht);
    Serial.print("Afstand:  ");
    Serial.println(totaldistance);
    Serial.println("");


    Serial.println("");*/
  Scheduler.delay(1000);
}

/*********************************************************************************************************
  THREADING LOOPS
*********************************************************************************************************/

void rotaryEncoder() {
  if (automatic == 0) {
    currentStateCLK = digitalRead(inputCLK);

    if (currentStateCLK != previousStateCLK) {
      if (digitalRead(inputDT) != currentStateCLK) {
        counter --;
        if (counter <= 0) {
          counter = 0;
        }
      }
      else {
        counter ++;
        if (counter >= 79) {
          counter = 79;
        }
      }
    }

    pwmDuty(counter); //Depends speed
    previousStateCLK = currentStateCLK;
  }

  else if (automatic == 1) {
    float autospeed = sq(curTemp - 28) * 79 / 289;
    counter = round(autospeed);
    //counter = 79;
    if (counter < 0) {
      counter = 0;
    }
    if (counter > 79) {
      counter = 79;
    }
    //Serial.println(String(autospeed) + " " + String(counter) + " " + String(curTemp));
    pwmDuty(counter); //Depends speed
  }

  Scheduler.delay(3);
}

void loopLight() {
  float waardeL1 = analogRead(A4);
  float waardeL2 = analogRead(A5);
  licht = lichtVerschil(waardeL1, waardeL2);
  Scheduler.delay(1000);
}

void distanceToScreen () {
  totaldistance = readdistance();
}
/*********************************************************************************************************
  FUNCTIONS
*********************************************************************************************************/

void pwm25kHzBegin() {
  TCCR2A = 0;                               // TC2 Control Register A
  TCCR2B = 0;                               // TC2 Control Register B
  TIMSK2 = 0;                               // TC2 Interrupt Mask Register
  TIFR2 = 0;                                // TC2 Interrupt Flag Register
  TCCR2A |= (1 << COM2B1) | (1 << WGM21) | (1 << WGM20);  // OC2B cleared/set on match when up/down counting, fast PWM
  TCCR2B |= (1 << WGM22) | (1 << CS21);     // prescaler 8
  OCR2A = 79;                               // TOP overflow value (Hz)
  OCR2B = 0;
}

void pwmDuty(byte ocrb) {
  OCR2B = ocrb;                             // PWM Width (duty)
}

int lichtVerschil(float input1, float input2) {
  float uitkomst1 =  input1 / 1023 * 100;
  float uitkomst2 =  input2 / 1023 * 100;
  float verschil = uitkomst1 - uitkomst2;
  if (verschil < 0) {
    return verschil * -1;
  }
  else {
    return verschil;
  }
}

int readdistance() {

  float Tdistance = 0;
  for (int i = 0; i < 99; i++) {
    digitalWrite(trigPin, LOW);
    delayMicroseconds(2);
    digitalWrite(trigPin, HIGH);
    delayMicroseconds(10);
    digitalWrite(trigPin, LOW);

    duration = pulseIn(echoPin, HIGH);
    distance = (duration * .0343) / 2;

    Tdistance += distance;
    Scheduler.delay(10);


  }
  distance = Tdistance / 99;

  return distance;
}

void fanActivation(float temp) {
  if (temp > minTemp) {
    digitalWrite(powerPin, HIGH);
  }
  else {
    digitalWrite(powerPin, LOW);
  }
}
